@@ -22,6 +22,15 @@ import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.Iterator;
+import java.util.Locale;
+import java.util.Set;
+import java.util.SortedMap;
+import java.util.TreeMap;
 
 import org.apache.commons.compress.archivers.ar.ArArchiveInputStream;
 import org.apache.commons.compress.archivers.ar.ArArchiveOutputStream;
@@ -37,6 +46,9 @@ import org.apache.commons.compress.archivers.tar.TarArchiveOutputStream;
 import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream;
 import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
 import org.apache.commons.compress.utils.IOUtils;
+import org.apache.commons.compress.utils.Lists;
+import org.apache.commons.compress.utils.ServiceLoaderIterator;
+import org.apache.commons.compress.utils.Sets;
 
 /**
  * Factory to create Archive[In|Out]putStreams from names or the first bytes of
@@ -74,45 +86,60 @@ import org.apache.commons.compress.utils.IOUtils;
  * @Immutable provided that the deprecated method setEntryEncoding is not used.
  * @ThreadSafe even if the deprecated method setEntryEncoding is used
  */
-public class ArchiveStreamFactory {
+public class ArchiveStreamFactory implements ArchiveStreamProvider {
 
+    private static final int TAR_HEADER_SIZE = 512;
+
+    private static final int DUMP_SIGNATURE_SIZE = 32;
+
+    private static final int SIGNATURE_SIZE = 12;
+
+    private static final ArchiveStreamFactory SINGLETON = new ArchiveStreamFactory();
+    
     /**
      * Constant (value {@value}) used to identify the AR archive format.
      * @since 1.1
      */
     public static final String AR = "ar";
+    
     /**
      * Constant (value {@value}) used to identify the ARJ archive format.
      * Not supported as an output stream type.
      * @since 1.6
      */
     public static final String ARJ = "arj";
+    
     /**
      * Constant (value {@value}) used to identify the CPIO archive format.
      * @since 1.1
      */
     public static final String CPIO = "cpio";
+    
     /**
      * Constant (value {@value}) used to identify the Unix DUMP archive format.
      * Not supported as an output stream type.
      * @since 1.3
      */
     public static final String DUMP = "dump";
+    
     /**
      * Constant (value {@value}) used to identify the JAR archive format.
      * @since 1.1
      */
     public static final String JAR = "jar";
+    
     /**
      * Constant used to identify the TAR archive format.
      * @since 1.1
      */
     public static final String TAR = "tar";
+    
     /**
      * Constant (value {@value}) used to identify the ZIP archive format.
      * @since 1.1
      */
     public static final String ZIP = "zip";
+    
     /**
      * Constant (value {@value}) used to identify the 7z archive format.
      * @since 1.8
@@ -127,7 +154,112 @@ public class ArchiveStreamFactory {
     /**
      * Entry encoding, null for the default.
      */
-    private volatile String entryEncoding = null;
+    private volatile String entryEncoding;
+
+    private SortedMap<String, ArchiveStreamProvider> archiveInputStreamProviders;
+
+    private SortedMap<String, ArchiveStreamProvider> archiveOutputStreamProviders;
+
+    private static ArrayList<ArchiveStreamProvider> findArchiveStreamProviders() {
+        return Lists.newArrayList(serviceLoaderIterator());
+    }
+
+    static void putAll(Set<String> names, ArchiveStreamProvider provider,
+            TreeMap<String, ArchiveStreamProvider> map) {
+        for (String name : names) {
+            map.put(toKey(name), provider);
+        }
+    }
+    
+    private static Iterator<ArchiveStreamProvider> serviceLoaderIterator() {
+        return new ServiceLoaderIterator<>(ArchiveStreamProvider.class);
+    }
+
+    private static String toKey(final String name) {
+        return name.toUpperCase(Locale.ROOT);
+    }
+
+    /**
+     * Constructs a new sorted map from input stream provider names to provider
+     * objects.
+     *
+     * <p>
+     * The map returned by this method will have one entry for each provider for
+     * which support is available in the current Java virtual machine. If two or
+     * more supported provider have the same name then the resulting map will
+     * contain just one of them; which one it will contain is not specified.
+     * </p>
+     *
+     * <p>
+     * The invocation of this method, and the subsequent use of the resulting
+     * map, may cause time-consuming disk or network I/O operations to occur.
+     * This method is provided for applications that need to enumerate all of
+     * the available providers, for example to allow user provider selection.
+     * </p>
+     *
+     * <p>
+     * This method may return different results at different times if new
+     * providers are dynamically made available to the current Java virtual
+     * machine.
+     * </p>
+     *
+     * @return An immutable, map from names to provider objects
+     * @since 1.13
+     */
+    public static SortedMap<String, ArchiveStreamProvider> findAvailableArchiveInputStreamProviders() {
+        return AccessController.doPrivileged(new PrivilegedAction<SortedMap<String, ArchiveStreamProvider>>() {
+            @Override
+            public SortedMap<String, ArchiveStreamProvider> run() {
+                TreeMap<String, ArchiveStreamProvider> map = new TreeMap<>();
+                putAll(SINGLETON.getInputStreamArchiveNames(), SINGLETON, map);
+                for (ArchiveStreamProvider provider : findArchiveStreamProviders()) {
+                    putAll(provider.getInputStreamArchiveNames(), provider, map);
+                }
+                return map;
+            }
+        });
+    }
+
+    /**
+     * Constructs a new sorted map from output stream provider names to provider
+     * objects.
+     *
+     * <p>
+     * The map returned by this method will have one entry for each provider for
+     * which support is available in the current Java virtual machine. If two or
+     * more supported provider have the same name then the resulting map will
+     * contain just one of them; which one it will contain is not specified.
+     * </p>
+     *
+     * <p>
+     * The invocation of this method, and the subsequent use of the resulting
+     * map, may cause time-consuming disk or network I/O operations to occur.
+     * This method is provided for applications that need to enumerate all of
+     * the available providers, for example to allow user provider selection.
+     * </p>
+     *
+     * <p>
+     * This method may return different results at different times if new
+     * providers are dynamically made available to the current Java virtual
+     * machine.
+     * </p>
+     *
+     * @return An immutable, map from names to provider objects
+     * @since 1.13
+     */
+    public static SortedMap<String, ArchiveStreamProvider> findAvailableArchiveOutputStreamProviders() {
+        return AccessController.doPrivileged(new PrivilegedAction<SortedMap<String, ArchiveStreamProvider>>() {
+            @Override
+            public SortedMap<String, ArchiveStreamProvider> run() {
+                TreeMap<String, ArchiveStreamProvider> map = new TreeMap<>();
+                putAll(SINGLETON.getOutputStreamArchiveNames(), SINGLETON, map);
+                for (ArchiveStreamProvider provider : findArchiveStreamProviders()) {
+                    putAll(provider.getOutputStreamArchiveNames(), provider, map);
+                }
+                return map;
+            }
+        });
+    }
 
     /**
      * Create an instance using the platform default encoding.
@@ -143,7 +275,7 @@ public class ArchiveStreamFactory {
      *
      * @since 1.10
      */
-    public ArchiveStreamFactory(String encoding) {
+    public ArchiveStreamFactory(final String encoding) {
         super();
         this.encoding = encoding;
         // Also set the original field so can continue to use it.
@@ -171,7 +303,7 @@ public class ArchiveStreamFactory {
      * was used to specify the factory encoding.
      */
     @Deprecated
-    public void setEntryEncoding(String entryEncoding) {
+    public void setEntryEncoding(final String entryEncoding) {
         // Note: this does not detect new ArchiveStreamFactory(null) but that does not set the encoding anyway
         if (encoding != null) {
             throw new IllegalStateException("Cannot overide encoding set by the constructor");
@@ -180,7 +312,7 @@ public class ArchiveStreamFactory {
     }
 
     /**
-     * Create an archive input stream from an archiver name and an input stream.
+     * Creates an archive input stream from an archiver name and an input stream.
      * 
      * @param archiverName the archive name,
      * i.e. {@value #AR}, {@value #ARJ}, {@value #ZIP}, {@value #TAR}, {@value #JAR}, {@value #CPIO}, {@value #DUMP} or {@value #SEVEN_Z}
@@ -191,9 +323,14 @@ public class ArchiveStreamFactory {
      * read from a stream
      * @throws IllegalArgumentException if the archiver name or stream is null
      */
-    public ArchiveInputStream createArchiveInputStream(
-            final String archiverName, final InputStream in)
+    public ArchiveInputStream createArchiveInputStream(final String archiverName, final InputStream in)
             throws ArchiveException {
+        return createArchiveInputStream(archiverName, in, entryEncoding);
+    }
+    
+    @Override
+    public ArchiveInputStream createArchiveInputStream(final String archiverName, final InputStream in,
+            final String actualEncoding) throws ArchiveException {
 
         if (archiverName == null) {
             throw new IllegalArgumentException("Archivername must not be null.");
@@ -207,56 +344,55 @@ public class ArchiveStreamFactory {
             return new ArArchiveInputStream(in);
         }
         if (ARJ.equalsIgnoreCase(archiverName)) {
-            if (entryEncoding != null) {
-                return new ArjArchiveInputStream(in, entryEncoding);
-            } else {
-                return new ArjArchiveInputStream(in);
+            if (actualEncoding != null) {
+                return new ArjArchiveInputStream(in, actualEncoding);
             }
+            return new ArjArchiveInputStream(in);
         }
         if (ZIP.equalsIgnoreCase(archiverName)) {
-            if (entryEncoding != null) {
-                return new ZipArchiveInputStream(in, entryEncoding);
-            } else {
-                return new ZipArchiveInputStream(in);
+            if (actualEncoding != null) {
+                return new ZipArchiveInputStream(in, actualEncoding);
             }
+            return new ZipArchiveInputStream(in);
         }
         if (TAR.equalsIgnoreCase(archiverName)) {
-            if (entryEncoding != null) {
-                return new TarArchiveInputStream(in, entryEncoding);
-            } else {
-                return new TarArchiveInputStream(in);
+            if (actualEncoding != null) {
+                return new TarArchiveInputStream(in, actualEncoding);
             }
+            return new TarArchiveInputStream(in);
         }
         if (JAR.equalsIgnoreCase(archiverName)) {
-            if (entryEncoding != null) {
-                return new JarArchiveInputStream(in, entryEncoding);
-            } else {
-                return new JarArchiveInputStream(in);
+            if (actualEncoding != null) {
+                return new JarArchiveInputStream(in, actualEncoding);
             }
+            return new JarArchiveInputStream(in);
         }
         if (CPIO.equalsIgnoreCase(archiverName)) {
-            if (entryEncoding != null) {
-                return new CpioArchiveInputStream(in, entryEncoding);
-            } else {
-                return new CpioArchiveInputStream(in);
+            if (actualEncoding != null) {
+                return new CpioArchiveInputStream(in, actualEncoding);
             }
+            return new CpioArchiveInputStream(in);
         }
         if (DUMP.equalsIgnoreCase(archiverName)) {
-            if (entryEncoding != null) {
-                return new DumpArchiveInputStream(in, entryEncoding);
-            } else {
-                return new DumpArchiveInputStream(in);
+            if (actualEncoding != null) {
+                return new DumpArchiveInputStream(in, actualEncoding);
             }
+            return new DumpArchiveInputStream(in);
         }
         if (SEVEN_Z.equalsIgnoreCase(archiverName)) {
             throw new StreamingNotSupportedException(SEVEN_Z);
         }
 
+        final ArchiveStreamProvider archiveStreamProvider = getArchiveInputStreamProviders().get(toKey(archiverName));
+        if (archiveStreamProvider != null) {
+            return archiveStreamProvider.createArchiveInputStream(archiverName, in, actualEncoding);
+        }
+
         throw new ArchiveException("Archiver: " + archiverName + " not found.");
     }
 
     /**
-     * Create an archive output stream from an archiver name and an output stream.
+     * Creates an archive output stream from an archiver name and an output stream.
      * 
      * @param archiverName the archive name,
      * i.e. {@value #AR}, {@value #ZIP}, {@value #TAR}, {@value #JAR} or {@value #CPIO} 
@@ -267,8 +403,14 @@ public class ArchiveStreamFactory {
      * written to a stream
      * @throws IllegalArgumentException if the archiver name or stream is null
      */
+    public ArchiveOutputStream createArchiveOutputStream(final String archiverName, final OutputStream out)
+            throws ArchiveException {
+        return createArchiveOutputStream(archiverName, out, entryEncoding);
+    }
+    
+    @Override
     public ArchiveOutputStream createArchiveOutputStream(
-            final String archiverName, final OutputStream out)
+            final String archiverName, final OutputStream out, final String actualEncoding)
             throws ArchiveException {
         if (archiverName == null) {
             throw new IllegalArgumentException("Archivername must not be null.");
@@ -281,36 +423,39 @@ public class ArchiveStreamFactory {
             return new ArArchiveOutputStream(out);
         }
         if (ZIP.equalsIgnoreCase(archiverName)) {
-            ZipArchiveOutputStream zip = new ZipArchiveOutputStream(out);
-            if (entryEncoding != null) {
-                zip.setEncoding(entryEncoding);
+            final ZipArchiveOutputStream zip = new ZipArchiveOutputStream(out);
+            if (actualEncoding != null) {
+                zip.setEncoding(actualEncoding);
             }
             return zip;
         }
         if (TAR.equalsIgnoreCase(archiverName)) {
-            if (entryEncoding != null) {
-                return new TarArchiveOutputStream(out, entryEncoding);
-            } else {
-                return new TarArchiveOutputStream(out);
+            if (actualEncoding != null) {
+                return new TarArchiveOutputStream(out, actualEncoding);
             }
+            return new TarArchiveOutputStream(out);
         }
         if (JAR.equalsIgnoreCase(archiverName)) {
-            if (entryEncoding != null) {
-                return new JarArchiveOutputStream(out, entryEncoding);
-            } else {
-                return new JarArchiveOutputStream(out);
+            if (actualEncoding != null) {
+                return new JarArchiveOutputStream(out, actualEncoding);
             }
+            return new JarArchiveOutputStream(out);
         }
         if (CPIO.equalsIgnoreCase(archiverName)) {
-            if (entryEncoding != null) {
-                return new CpioArchiveOutputStream(out, entryEncoding);
-            } else {
-                return new CpioArchiveOutputStream(out);
+            if (actualEncoding != null) {
+                return new CpioArchiveOutputStream(out, actualEncoding);
             }
+            return new CpioArchiveOutputStream(out);
         }
         if (SEVEN_Z.equalsIgnoreCase(archiverName)) {
             throw new StreamingNotSupportedException(SEVEN_Z);
         }
+
+        final ArchiveStreamProvider archiveStreamProvider = getArchiveOutputStreamProviders().get(toKey(archiverName));
+        if (archiveStreamProvider != null) {
+            return archiveStreamProvider.createArchiveOutputStream(archiverName, out, actualEncoding);
+        }
+
         throw new ArchiveException("Archiver: " + archiverName + " not found.");
     }
 
@@ -336,7 +481,7 @@ public class ArchiveStreamFactory {
             throw new IllegalArgumentException("Mark is not supported.");
         }
 
-        final byte[] signature = new byte[12];
+        final byte[] signature = new byte[SIGNATURE_SIZE];
         in.mark(signature.length);
         try {
             int signatureLength = IOUtils.readFully(in, signature);
@@ -356,7 +501,7 @@ public class ArchiveStreamFactory {
             }
 
             // Dump needs a bigger buffer to check the signature;
-            final byte[] dumpsig = new byte[32];
+            final byte[] dumpsig = new byte[DUMP_SIGNATURE_SIZE];
             in.mark(dumpsig.length);
             signatureLength = IOUtils.readFully(in, dumpsig);
             in.reset();
@@ -365,23 +510,23 @@ public class ArchiveStreamFactory {
             }
 
             // Tar needs an even bigger buffer to check the signature; read the first block
-            final byte[] tarheader = new byte[512];
-            in.mark(tarheader.length);
-            signatureLength = IOUtils.readFully(in, tarheader);
+            final byte[] tarHeader = new byte[TAR_HEADER_SIZE];
+            in.mark(tarHeader.length);
+            signatureLength = IOUtils.readFully(in, tarHeader);
             in.reset();
-            if (TarArchiveInputStream.matches(tarheader, signatureLength)) {
+            if (TarArchiveInputStream.matches(tarHeader, signatureLength)) {
                 return createArchiveInputStream(TAR, in);
             }
             // COMPRESS-117 - improve auto-recognition
-            if (signatureLength >= 512) {
+            if (signatureLength >= TAR_HEADER_SIZE) {
                 TarArchiveInputStream tais = null;
                 try {
-                    tais = new TarArchiveInputStream(new ByteArrayInputStream(tarheader));
+                    tais = new TarArchiveInputStream(new ByteArrayInputStream(tarHeader));
                     // COMPRESS-191 - verify the header checksum
                     if (tais.getNextTarEntry().isCheckSumOK()) {
                         return createArchiveInputStream(TAR, in);
                     }
-                } catch (Exception e) { // NOPMD
+                } catch (final Exception e) { // NOPMD
                     // can generate IllegalArgumentException as well
                     // as IOException
                     // autodetection, simply not a TAR
@@ -390,11 +535,37 @@ public class ArchiveStreamFactory {
                     IOUtils.closeQuietly(tais);
                 }
             }
-        } catch (IOException e) {
+        } catch (final IOException e) {
             throw new ArchiveException("Could not use reset and mark operations.", e);
         }
 
         throw new ArchiveException("No Archiver found for the stream signature");
     }
 
+    public SortedMap<String, ArchiveStreamProvider> getArchiveInputStreamProviders() {
+        if (archiveInputStreamProviders == null) {
+            archiveInputStreamProviders = Collections
+                    .unmodifiableSortedMap(findAvailableArchiveInputStreamProviders());
+        }
+        return archiveInputStreamProviders;
+    }
+
+    public SortedMap<String, ArchiveStreamProvider> getArchiveOutputStreamProviders() {
+        if (archiveOutputStreamProviders == null) {
+            archiveOutputStreamProviders = Collections
+                    .unmodifiableSortedMap(findAvailableArchiveOutputStreamProviders());
+        }
+        return archiveOutputStreamProviders;
+    }
+
+    @Override
+    public Set<String> getInputStreamArchiveNames() {
+        return Sets.newHashSet(AR, ARJ, ZIP, TAR, JAR, CPIO, DUMP, SEVEN_Z);
+    }
+
+    @Override
+    public Set<String> getOutputStreamArchiveNames() {
+        return Sets.newHashSet(AR, ZIP, TAR, JAR, CPIO, SEVEN_Z);
+    }
+
 }
