@@ -49,7 +49,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
 
     private static final int SMALL_BUFFER_SIZE = 256;
 
-    private final byte[] SMALL_BUF = new byte[SMALL_BUFFER_SIZE];
+    private final byte[] smallBuf = new byte[SMALL_BUFFER_SIZE];
 
     /** The size the TAR header */
     private final int recordSize;
@@ -79,13 +79,13 @@ public class TarArchiveInputStream extends ArchiveInputStream {
     final String encoding;
 
     // the global PAX header
-    private Map<String, String> globalPaxHeaders = new HashMap<String, String>();
+    private Map<String, String> globalPaxHeaders = new HashMap<>();
 
     /**
      * Constructor for TarInputStream.
      * @param is the input stream to use
      */
-    public TarArchiveInputStream(InputStream is) {
+    public TarArchiveInputStream(final InputStream is) {
         this(is, TarConstants.DEFAULT_BLKSIZE, TarConstants.DEFAULT_RCDSIZE);
     }
 
@@ -95,7 +95,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @param encoding name of the encoding to use for file names
      * @since 1.4
      */
-    public TarArchiveInputStream(InputStream is, String encoding) {
+    public TarArchiveInputStream(final InputStream is, final String encoding) {
         this(is, TarConstants.DEFAULT_BLKSIZE, TarConstants.DEFAULT_RCDSIZE,
              encoding);
     }
@@ -105,7 +105,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @param is the input stream to use
      * @param blockSize the block size to use
      */
-    public TarArchiveInputStream(InputStream is, int blockSize) {
+    public TarArchiveInputStream(final InputStream is, final int blockSize) {
         this(is, blockSize, TarConstants.DEFAULT_RCDSIZE);
     }
 
@@ -116,8 +116,8 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @param encoding name of the encoding to use for file names
      * @since 1.4
      */
-    public TarArchiveInputStream(InputStream is, int blockSize,
-                                 String encoding) {
+    public TarArchiveInputStream(final InputStream is, final int blockSize,
+                                 final String encoding) {
         this(is, blockSize, TarConstants.DEFAULT_RCDSIZE, encoding);
     }
 
@@ -127,7 +127,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @param blockSize the block size to use
      * @param recordSize the record size to use
      */
-    public TarArchiveInputStream(InputStream is, int blockSize, int recordSize) {
+    public TarArchiveInputStream(final InputStream is, final int blockSize, final int recordSize) {
         this(is, blockSize, recordSize, null);      
     }
 
@@ -139,8 +139,8 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @param encoding name of the encoding to use for file names
      * @since 1.4
      */
-    public TarArchiveInputStream(InputStream is, int blockSize, int recordSize,
-                                 String encoding) {
+    public TarArchiveInputStream(final InputStream is, final int blockSize, final int recordSize,
+                                 final String encoding) {
         this.is = is;
         this.hasHitEOF = false;
         this.encoding = encoding;
@@ -204,7 +204,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @param n
      *            the number of bytes to be skipped.
      * @return the actual number of bytes skipped.
-     * @exception IOException
+     * @throws IOException
      *                if some other I/O error occurs.
      */
     @Override
@@ -236,7 +236,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @param markLimit The limit to mark.
      */
     @Override
-    public void mark(int markLimit) {
+    public void mark(final int markLimit) {
     }
 
     /**
@@ -272,7 +272,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
             skipRecordPadding();
         }
 
-        byte[] headerBuf = getRecord();
+        final byte[] headerBuf = getRecord();
 
         if (headerBuf == null) {
             /* hit EOF */
@@ -282,17 +282,15 @@ public class TarArchiveInputStream extends ArchiveInputStream {
 
         try {
             currEntry = new TarArchiveEntry(headerBuf, zipEncoding);
-        } catch (IllegalArgumentException e) {
-            IOException ioe = new IOException("Error detected parsing the header");
-            ioe.initCause(e);
-            throw ioe;
+        } catch (final IllegalArgumentException e) {
+            throw new IOException("Error detected parsing the header", e);
         }
 
         entryOffset = 0;
         entrySize = currEntry.getSize();
 
         if (currEntry.isGNULongLinkEntry()) {
-            byte[] longLinkData = getLongNameData();
+            final byte[] longLinkData = getLongNameData();
             if (longLinkData == null) {
                 // Bugzilla: 40334
                 // Malformed tar file - long link entry name not followed by
@@ -303,7 +301,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
         }
 
         if (currEntry.isGNULongNameEntry()) {
-            byte[] longNameData = getLongNameData();
+            final byte[] longNameData = getLongNameData();
             if (longNameData == null) {
                 // Bugzilla: 40334
                 // Malformed tar file - long entry name not followed by
@@ -342,9 +340,9 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      */
     private void skipRecordPadding() throws IOException {
         if (!isDirectory() && this.entrySize > 0 && this.entrySize % this.recordSize != 0) {
-            long numRecords = (this.entrySize / this.recordSize) + 1;
-            long padding = (numRecords * this.recordSize) - this.entrySize;
-            long skipped = IOUtils.skip(is, padding);
+            final long numRecords = (this.entrySize / this.recordSize) + 1;
+            final long padding = (numRecords * this.recordSize) - this.entrySize;
+            final long skipped = IOUtils.skip(is, padding);
             count(skipped);
         }
     }
@@ -357,10 +355,10 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      */
     protected byte[] getLongNameData() throws IOException {
         // read in the name
-        ByteArrayOutputStream longName = new ByteArrayOutputStream();
+        final ByteArrayOutputStream longName = new ByteArrayOutputStream();
         int length = 0;
-        while ((length = read(SMALL_BUF)) >= 0) {
-            longName.write(SMALL_BUF, 0, length);
+        while ((length = read(smallBuf)) >= 0) {
+            longName.write(smallBuf, 0, length);
         }
         getNextEntry();
         if (currEntry == null) {
@@ -375,7 +373,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
             --length;
         }
         if (length != longNameData.length) {
-            byte[] l = new byte[length];
+            final byte[] l = new byte[length];
             System.arraycopy(longNameData, 0, l, 0, length);
             longNameData = l;
         }
@@ -414,7 +412,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @param record The record data to check.
      * @return true if the record data is an End of Archive
      */
-    protected boolean isEOFRecord(byte[] record) {
+    protected boolean isEOFRecord(final byte[] record) {
         return record == null || ArchiveUtils.isArrayZero(record, recordSize);
     }
     
@@ -426,9 +424,9 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      */
     protected byte[] readRecord() throws IOException {
 
-        byte[] record = new byte[recordSize];
+        final byte[] record = new byte[recordSize];
 
-        int readNow = IOUtils.readFully(is, record);
+        final int readNow = IOUtils.readFully(is, record);
         count(readNow);
         if (readNow != recordSize) {
             return null;
@@ -443,7 +441,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
     }
 
     private void paxHeaders() throws IOException{
-        Map<String, String> headers = parsePaxHeaders(this);
+        final Map<String, String> headers = parsePaxHeaders(this);
         getNextEntry(); // Get the actual file entry
         applyPaxHeadersToCurrentEntry(headers);
     }
@@ -451,9 +449,9 @@ public class TarArchiveInputStream extends ArchiveInputStream {
     // NOTE, using a Map here makes it impossible to ever support GNU
     // sparse files using the PAX Format 0.0, see
     // https://www.gnu.org/software/tar/manual/html_section/tar_92.html#SEC188
-    Map<String, String> parsePaxHeaders(InputStream i)
+    Map<String, String> parsePaxHeaders(final InputStream i)
         throws IOException {
-        Map<String, String> headers = new HashMap<String, String>(globalPaxHeaders);
+        final Map<String, String> headers = new HashMap<>(globalPaxHeaders);
         // Format is "length keyword=value\n";
         while(true){ // get length
             int ch;
@@ -461,20 +459,22 @@ public class TarArchiveInputStream extends ArchiveInputStream {
             int read = 0;
             while((ch = i.read()) != -1) {
                 read++;
-                if (ch == ' '){ // End of length string
+                if (ch == '\n') { // blank line in header
+                    break;
+                } else if (ch == ' '){ // End of length string
                     // Get keyword
-                    ByteArrayOutputStream coll = new ByteArrayOutputStream();
+                    final ByteArrayOutputStream coll = new ByteArrayOutputStream();
                     while((ch = i.read()) != -1) {
                         read++;
                         if (ch == '='){ // end of keyword
-                            String keyword = coll.toString(CharsetNames.UTF_8);
+                            final String keyword = coll.toString(CharsetNames.UTF_8);
                             // Get rest of entry
                             final int restLen = len - read;
                             if (restLen == 1) { // only NL
                                 headers.remove(keyword);
                             } else {
-                                byte[] rest = new byte[restLen];
-                                int got = IOUtils.readFully(i, rest);
+                                final byte[] rest = new byte[restLen];
+                                final int got = IOUtils.readFully(i, rest);
                                 if (got != restLen) {
                                     throw new IOException("Failed to read "
                                                           + "Paxheader. Expected "
@@ -483,7 +483,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
                                                           + got);
                                 }
                                 // Drop trailing NL
-                                String value = new String(rest, 0,
+                                final String value = new String(rest, 0,
                                                           restLen - 1, CharsetNames.UTF_8);
                                 headers.put(keyword, value);
                             }
@@ -503,7 +503,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
         return headers;
     }
 
-    private void applyPaxHeadersToCurrentEntry(Map<String, String> headers) {
+    private void applyPaxHeadersToCurrentEntry(final Map<String, String> headers) {
         /*
          * The following headers are defined for Pax.
          * atime, ctime, charset: cannot use these without changing TarArchiveEntry fields
@@ -522,9 +522,9 @@ public class TarArchiveInputStream extends ArchiveInputStream {
          * star files use additional members of which we use
          * SCHILY.filetype in order to detect star sparse files.
          */
-        for (Entry<String, String> ent : headers.entrySet()){
-            String key = ent.getKey();
-            String val = ent.getValue();
+        for (final Entry<String, String> ent : headers.entrySet()){
+            final String key = ent.getKey();
+            final String val = ent.getValue();
             if ("path".equals(key)){
                 currEntry.setName(val);
             } else if ("linkpath".equals(key)){
@@ -571,7 +571,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
         if (currEntry.isExtended()) {
             TarArchiveSparseEntry entry;
             do {
-                byte[] headerBuf = getRecord();
+                final byte[] headerBuf = getRecord();
                 if (headerBuf == null) {
                     currEntry = null;
                     break;
@@ -612,7 +612,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      */
     private void tryToConsumeSecondEOFRecord() throws IOException {
         boolean shouldReset = true;
-        boolean marked = is.markSupported();
+        final boolean marked = is.markSupported();
         if (marked) {
             is.mark(recordSize);
         }
@@ -640,7 +640,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @throws IOException on error
      */
     @Override
-    public int read(byte[] buf, int offset, int numToRead) throws IOException {
+    public int read(final byte[] buf, final int offset, int numToRead) throws IOException {
     	int totalRead = 0;
 
         if (hasHitEOF || isDirectory() || entryOffset >= entrySize) {
@@ -674,9 +674,9 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * <p>May return false if the current entry is a sparse file.</p>
      */
     @Override
-    public boolean canReadEntryData(ArchiveEntry ae) {
+    public boolean canReadEntryData(final ArchiveEntry ae) {
         if (ae instanceof TarArchiveEntry) {
-            TarArchiveEntry te = (TarArchiveEntry) ae;
+            final TarArchiveEntry te = (TarArchiveEntry) ae;
             return !te.isSparse();
         }
         return false;
@@ -691,7 +691,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
         return currEntry;
     }
 
-    protected final void setCurrentEntry(TarArchiveEntry e) {
+    protected final void setCurrentEntry(final TarArchiveEntry e) {
         currEntry = e;
     }
 
@@ -699,7 +699,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
         return hasHitEOF;
     }
 
-    protected final void setAtEOF(boolean b) {
+    protected final void setAtEOF(final boolean b) {
         hasHitEOF = b;
     }
 
@@ -709,9 +709,9 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * the tool creating this archive has padded the last block.
      */
     private void consumeRemainderOfLastBlock() throws IOException {
-        long bytesReadOfLastBlock = getBytesRead() % blockSize;
+        final long bytesReadOfLastBlock = getBytesRead() % blockSize;
         if (bytesReadOfLastBlock > 0) {
-            long skipped = IOUtils.skip(is, blockSize - bytesReadOfLastBlock);
+            final long skipped = IOUtils.skip(is, blockSize - bytesReadOfLastBlock);
             count(skipped);
         }
     }
@@ -725,7 +725,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      *            the number of bytes to check
      * @return true, if this stream is a tar archive stream, false otherwise
      */
-    public static boolean matches(byte[] signature, int length) {
+    public static boolean matches(final byte[] signature, final int length) {
         if (length < TarConstants.VERSION_OFFSET+TarConstants.VERSIONLEN) {
             return false;
         }
